#include <stdlib.h>
#include <pic.h>
#include <xc.h>

#define _XTAL_FREQ 1000000
#define AN3  1<<3

// PIC12F675 Configuration Bit Settings
#pragma config FOSC = INTRCIO // Oscillator Selection bits (INTOSC oscillator: I/O function on GP4/OSC2/CLKOUT pin, I/O function on GP5/OSC1/CLKIN)
#pragma config WDTE = OFF     // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON     // Power-Up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = OFF    // GP3/MCLR pin function select (GP3/MCLR pin function is digital I/O, MCLR internally tied to VDD)
#pragma config BOREN = OFF    // Brown-out Detect Enable bit (BOD disabled)
#pragma config CP = OFF       // Code Protection bit (Program Memory code protection is disabled)
#pragma config CPD = OFF      // Data Code Protection bit (Data memory code protection is disabled)


void tone(unsigned int period_ms, unsigned int note_length_ms)
{
    unsigned int i;
    unsigned int note_length_periods = note_length_ms/period_ms;
    unsigned int half_period_ms = period_ms << 1;

    for(i = 0; i < note_length_periods; i++)
    {
        unsigned int j;

        GPIO = 0b00100101; // set GP0, GP2 & GP5 high

        for(j = 0; j < half_period_ms; j++)
        {
            __delay_ms(1);
        }

        GPIO = 0b00100100;

        for(j = 0; j < half_period_ms; j++)
        {
            __delay_ms(1);
        }
    }
}

void sweep(unsigned int start, unsigned int end, unsigned int interval)
{
    unsigned int i;

    if(start >= end)
    {
        for(i = start; i > end; i--)
        {
            tone(i, interval);
        }
    }

    if(end > start)
    {
        for(i = start; i < end; i++)
        {
            tone(i, interval);
        }
    }
}

void gap(unsigned int length_ms)
{
    unsigned int i;

    for(i = 0; i < length_ms; i++)
    {
        __delay_ms(1);
    }

}

void noise()
{
    unsigned int r = rand() % 6;

    tone(r, r);
    gap(r);
    tone(r, r);
    gap(r);
}

void InitADC(unsigned char Channel)
{
    ANSEL   = 0x10;	     // Clear Pin selection bits
    ANSEL  |= Channel;   // Select Channel
    TRISIO  = 0x10;      // GP3 input, rest all output
    TRISIO |= Channel;	 // Make selected channel pins input
    ADCON0 = 0x81;		 // Turn on the A/D Converter
    CMCON  = 0x07;		 // Shut off the Comparator, so that pins are available for ADC
    VRCON  = 0x00;	     // Shut off the Voltage Reference for Comparator
}

unsigned int GetADCValue(unsigned char Channel)
{
    ADCON0 &= 0xf3;      // Clear Channel selection bits
    ADCON0 |= 0x0c; break;      // Select GP4 pin as ADC input

    __delay_ms(10);      // Time for Acqusition capacitor to charge up and show correct value

    GO_nDONE  = 1;		 // Enable Go/Done
    while(GO_nDONE);     //wait for conversion completion

    return ((ADRESH<<8)+ADRESL);   // Return 10 bit ADC value
}

int main(int argc, char** argv)
{
    InitADC(AN3);
    GPIO = 0;

    unsigned int tempo_ms_interval = 1000;
    unsigned int i;

    while(1) // Loop forever
    {
        //unsigned int current_ADC_value = GetADCValue(AN3);

        rand = rand() % 4;

        for(i = 0; i < rand; i++)
        {
            tone(12, tempo_ms_interval/rand);
            gap(tempo_ms_interval/rand);
        }
    }
    
    return (EXIT_SUCCESS);
}